@@ -47,7 +47,6 @@
 #include "empathy-avatar-chooser.h"
 #include "empathy-avatar-image.h"
 #include "empathy-ui-utils.h"
-#include "empathy-string-parser.h"
 #include "empathy-kludge-label.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
@@ -949,15 +948,8 @@ contact_widget_name_notify_cb (EmpathyContactWidget *information)
 static void
 contact_widget_presence_notify_cb (EmpathyContactWidget *information)
 {
-  const gchar *status;
-  gchar *markup_text = NULL;
-
-  status = empathy_contact_get_status (information->contact);
-  if (status != NULL)
-    markup_text = empathy_add_link_markup (status);
-  gtk_label_set_markup (GTK_LABEL (information->label_status), markup_text);
-  g_free (markup_text);
-
+  gtk_label_set_text (GTK_LABEL (information->label_status),
+      empathy_contact_get_status (information->contact));
   gtk_image_set_from_icon_name (GTK_IMAGE (information->image_state),
       empathy_icon_name_for_contact (information->contact),
       GTK_ICON_SIZE_BUTTON);
