@@ -291,12 +291,6 @@ account_assistant_account_enabled_cb (GObject *source,
   protocol = tp_account_get_protocol (account);
   if (!tp_strdiff (protocol, "local-xmpp"))
     {
-      DEBUG ("Salut account has been created; update gconf key");
-
-      empathy_conf_set_bool (empathy_conf_get (),
-          EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
-          TRUE);
-
       salut_created = TRUE;
     }
 
@@ -348,21 +342,26 @@ account_assistant_apply_account_and_finish (EmpathyAccountAssistant *self,
     EmpathyAccountSettings *settings)
 {
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
-  gchar *display_name;
 
   if (settings == NULL)
     return;
 
   priv->is_creating = TRUE;
 
-  /* set default display name */
-  display_name = empathy_account_widget_get_default_display_name (
-      priv->current_widget_object);
+  /* set default display name, if there is no current widget then assume the
+   * display name was already set correctly. e.g. salut account creation */
+  if (priv->current_widget_object != NULL)
+    {
+      gchar *display_name;
 
-  empathy_account_settings_set_display_name_async (settings,
-      display_name, NULL, NULL);
+      display_name = empathy_account_widget_get_default_display_name (
+        priv->current_widget_object);
 
-  g_free (display_name);
+      empathy_account_settings_set_display_name_async (settings,
+        display_name, NULL, NULL);
+
+      g_free (display_name);
+    }
 
   empathy_account_settings_apply_async (settings,
       account_assistant_apply_account_cb, self);
@@ -417,11 +416,19 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
   settings = empathy_account_settings_new (cm->name, proto->name, str);
 
   if (is_gtalk)
-    empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
-      NULL, NULL);
+    {
+      empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
+        NULL, NULL);
+      /* We should not have to set the server but that may cause issue with
+       * buggy router. */
+      empathy_account_settings_set_string (settings, "server",
+          "talk.google.com");
+    }
   else if (is_facebook)
-    empathy_account_settings_set_icon_name_async (settings, "im-facebook",
-      NULL, NULL);
+    {
+      empathy_account_settings_set_icon_name_async (settings, "im-facebook",
+        NULL, NULL);
+    }
 
   if (priv->first_resp == RESPONSE_CREATE_ACCOUNT)
     empathy_account_settings_set_boolean (settings, "register", TRUE);
