@@ -28,6 +28,8 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
 
@@ -38,6 +40,10 @@
 #include <libempathy-gtk/empathy-contact-list-store.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
 
+#ifdef HAVE_WEBKIT
+#include <libempathy-gtk/empathy-theme-adium.h>
+#endif
+
 #include "empathy-preferences.h"
 
 typedef struct {
@@ -45,14 +51,11 @@ typedef struct {
 
 	GtkWidget *notebook;
 
-	GtkWidget *checkbutton_show_avatars;
-	GtkWidget *checkbutton_compact_contact_list;
 	GtkWidget *checkbutton_show_smileys;
+	GtkWidget *checkbutton_show_contacts_in_rooms;
 	GtkWidget *combobox_chat_theme;
 	GtkWidget *checkbutton_separate_chat_windows;
 	GtkWidget *checkbutton_autoconnect;
-	GtkWidget *radiobutton_contact_list_sort_by_name;
-	GtkWidget *radiobutton_contact_list_sort_by_state;
 
 	GtkWidget *checkbutton_sounds_enabled;
 	GtkWidget *checkbutton_sounds_disabled_away;
@@ -61,9 +64,17 @@ typedef struct {
 	GtkWidget *checkbutton_notifications_enabled;
 	GtkWidget *checkbutton_notifications_disabled_away;
 	GtkWidget *checkbutton_notifications_focus;
+	GtkWidget *checkbutton_notifications_contact_signin;
+	GtkWidget *checkbutton_notifications_contact_signout;
 
 	GtkWidget *treeview_spell_checker;
 
+	GtkWidget *checkbutton_location_publish;
+	GtkWidget *checkbutton_location_reduce_accuracy;
+	GtkWidget *checkbutton_location_resource_network;
+	GtkWidget *checkbutton_location_resource_cell;
+	GtkWidget *checkbutton_location_resource_gps;
+
 	GList     *notify_ids;
 } EmpathyPreferences;
 
@@ -83,19 +94,8 @@ static gboolean preferences_languages_load_foreach       (GtkTreeModel
 static void     preferences_languages_cell_toggled_cb    (GtkCellRendererToggle  *cell,
 							  gchar                  *path_string,
 							  EmpathyPreferences      *preferences);
-static void     preferences_themes_setup                 (EmpathyPreferences      *preferences);
 static void     preferences_widget_sync_bool             (const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_widget_sync_string           (const gchar            *key,
-							  GtkWidget              *widget);
-static void     preferences_widget_sync_string_combo     (const gchar            *key,
-							  GtkWidget              *widget);
-static void     preferences_notify_string_cb             (EmpathyConf             *conf,
-							  const gchar            *key,
-							  gpointer                user_data);
-static void     preferences_notify_string_combo_cb       (EmpathyConf             *conf,
-							  const gchar            *key,
-							  gpointer                user_data);
 static void     preferences_notify_bool_cb               (EmpathyConf             *conf,
 							  const gchar            *key,
 							  gpointer                user_data);
@@ -105,21 +105,11 @@ static void     preferences_notify_sensitivity_cb        (EmpathyConf
 static void     preferences_hookup_toggle_button         (EmpathyPreferences      *preferences,
 							  const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_hookup_radio_button          (EmpathyPreferences      *preferences,
-							  const gchar            *key,
-							  GtkWidget              *widget);
-static void     preferences_hookup_string_combo          (EmpathyPreferences      *preferences,
-							  const gchar            *key,
-							  GtkWidget              *widget);
 static void     preferences_hookup_sensitivity           (EmpathyPreferences      *preferences,
 							  const gchar            *key,
 							  GtkWidget              *widget);
 static void     preferences_toggle_button_toggled_cb     (GtkWidget              *button,
 							  gpointer                user_data);
-static void     preferences_radio_button_toggled_cb      (GtkWidget              *button,
-							  gpointer                user_data);
-static void     preferences_string_combo_changed_cb      (GtkWidget *button,
-							  gpointer                user_data);
 static void     preferences_destroy_cb                   (GtkWidget              *widget,
 							  EmpathyPreferences      *preferences);
 static void     preferences_response_cb                  (GtkWidget              *widget,
@@ -134,8 +124,10 @@ enum {
 };
 
 enum {
+	COL_COMBO_IS_ADIUM,
 	COL_COMBO_VISIBLE_NAME,
 	COL_COMBO_NAME,
+	COL_COMBO_PATH,
 	COL_COMBO_COUNT
 };
 
@@ -169,25 +161,9 @@ preferences_add_id (EmpathyPreferences *preferences, guint id)
 						  GUINT_TO_POINTER (id));
 }
 
-static void
-preferences_compact_contact_list_changed_cb (EmpathyConf *conf,
-					     const gchar *key,
-					     gpointer     user_data)
-{
-	EmpathyPreferences *preferences = user_data;
-	gboolean            value;
-
-	if (empathy_conf_get_bool (empathy_conf_get (), key, &value)) {
-		gtk_widget_set_sensitive (preferences->checkbutton_show_avatars,
-					  !value);
-	}
-}
-
 static void
 preferences_setup_widgets (EmpathyPreferences *preferences)
 {
-	guint id;
-
 	preferences_hookup_toggle_button (preferences,
 					  EMPATHY_PREFS_NOTIFICATIONS_ENABLED,
 					  preferences->checkbutton_notifications_enabled);
@@ -197,6 +173,12 @@ preferences_setup_widgets (EmpathyPreferences *preferences)
 	preferences_hookup_toggle_button (preferences,
 					  EMPATHY_PREFS_NOTIFICATIONS_FOCUS,
 					  preferences->checkbutton_notifications_focus);
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN,
+					  preferences->checkbutton_notifications_contact_signin);
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT,
+					  preferences->checkbutton_notifications_contact_signout);
 
 	preferences_hookup_sensitivity (preferences,
 					EMPATHY_PREFS_NOTIFICATIONS_ENABLED,
@@ -204,6 +186,12 @@ preferences_setup_widgets (EmpathyPreferences *preferences)
 	preferences_hookup_sensitivity (preferences,
 					EMPATHY_PREFS_NOTIFICATIONS_ENABLED,
 					preferences->checkbutton_notifications_focus);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_NOTIFICATIONS_ENABLED,
+					preferences->checkbutton_notifications_contact_signin);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_NOTIFICATIONS_ENABLED,
+					preferences->checkbutton_notifications_contact_signout);
 
 	preferences_hookup_toggle_button (preferences,
 					  EMPATHY_PREFS_SOUNDS_ENABLED,
@@ -223,40 +211,49 @@ preferences_setup_widgets (EmpathyPreferences *preferences)
 					  EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
 					  preferences->checkbutton_separate_chat_windows);
 
-	preferences_hookup_toggle_button (preferences,
-					  EMPATHY_PREFS_UI_SHOW_AVATARS,
-					  preferences->checkbutton_show_avatars);
-
-	preferences_hookup_toggle_button (preferences,
-					  EMPATHY_PREFS_UI_COMPACT_CONTACT_LIST,
-					  preferences->checkbutton_compact_contact_list);
-
 	preferences_hookup_toggle_button (preferences,
 					  EMPATHY_PREFS_CHAT_SHOW_SMILEYS,
 					  preferences->checkbutton_show_smileys);
 
-	preferences_hookup_string_combo (preferences,
-					 EMPATHY_PREFS_CHAT_THEME,
-					 preferences->combobox_chat_theme);
-
-	preferences_hookup_radio_button (preferences,
-					 EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM,
-					 preferences->radiobutton_contact_list_sort_by_name);
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_CHAT_SHOW_CONTACTS_IN_ROOMS,
+					  preferences->checkbutton_show_contacts_in_rooms);
 
 	preferences_hookup_toggle_button (preferences,
 					  EMPATHY_PREFS_AUTOCONNECT,
 					  preferences->checkbutton_autoconnect);
 
-	id = empathy_conf_notify_add (empathy_conf_get (),
-				      EMPATHY_PREFS_UI_COMPACT_CONTACT_LIST,
-				      preferences_compact_contact_list_changed_cb,
-				      preferences);
-	if (id) {
-		preferences_add_id (preferences, id);
-	}
-	preferences_compact_contact_list_changed_cb (empathy_conf_get (),
-						     EMPATHY_PREFS_UI_COMPACT_CONTACT_LIST,
-						     preferences);
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_LOCATION_PUBLISH,
+					  preferences->checkbutton_location_publish);
+
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_LOCATION_RESOURCE_NETWORK,
+					  preferences->checkbutton_location_resource_network);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_LOCATION_PUBLISH,
+					preferences->checkbutton_location_resource_network);
+
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_LOCATION_RESOURCE_CELL,
+					  preferences->checkbutton_location_resource_cell);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_LOCATION_PUBLISH,
+					preferences->checkbutton_location_resource_cell);
+
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_LOCATION_RESOURCE_GPS,
+					  preferences->checkbutton_location_resource_gps);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_LOCATION_PUBLISH,
+					preferences->checkbutton_location_resource_gps);
+
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_LOCATION_REDUCE_ACCURACY,
+					  preferences->checkbutton_location_reduce_accuracy);
+	preferences_hookup_sensitivity (preferences,
+					EMPATHY_PREFS_LOCATION_PUBLISH,
+					preferences->checkbutton_location_reduce_accuracy);
 }
 
 static void
@@ -295,7 +292,7 @@ preferences_sound_cell_toggled_cb (GtkCellRendererToggle *toggle,
 static void
 preferences_sound_load (EmpathyPreferences *preferences)
 {
-	int i;
+	guint i;
 	GtkTreeView *view;
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -427,7 +424,7 @@ preferences_languages_add (EmpathyPreferences *preferences)
 			       codes != NULL);
 	if (!codes) {
 		gtk_widget_set_sensitive (preferences->treeview_spell_checker, FALSE);
-	}		
+	}
 
 	for (l = codes; l; l = l->next) {
 		GtkTreeIter  iter;
@@ -561,7 +558,7 @@ preferences_languages_load_foreach (GtkTreeModel  *model,
 	}
 
 	for (i = 0, lang = languages[i]; lang; lang = languages[++i]) {
-		if (strcmp (lang, code) == 0) {
+		if (!tp_strdiff (lang, code)) {
 			found = TRUE;
 		}
 	}
@@ -600,43 +597,6 @@ preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell,
 	preferences_languages_save (preferences);
 }
 
-static void
-preferences_themes_setup (EmpathyPreferences *preferences)
-{
-	GtkComboBox   *combo;
-	GtkCellLayout *cell_layout;
-	GtkCellRenderer *renderer;
-	GtkListStore  *store;
-	const gchar  **themes;
-	gint           i;
-
-	combo = GTK_COMBO_BOX (preferences->combobox_chat_theme);
-	cell_layout = GTK_CELL_LAYOUT (combo);
-
-	/* Create the model */
-	store = gtk_list_store_new (COL_COMBO_COUNT,
-				    G_TYPE_STRING,  /* Display name */
-				    G_TYPE_STRING); /* Theme name */
-
-	/* Fill the model */
-	themes = empathy_theme_manager_get_themes ();
-	for (i = 0; themes[i]; i += 2) {
-		gtk_list_store_insert_with_values (store, NULL, -1,
-			COL_COMBO_VISIBLE_NAME, _(themes[i + 1]),
-			COL_COMBO_NAME, themes[i],
-			-1);
-	}
-
-	/* Add cell renderer */
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (cell_layout, renderer, TRUE);
-	gtk_cell_layout_set_attributes (cell_layout, renderer,
-		"text", COL_COMBO_VISIBLE_NAME, NULL);
-
-	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
-	g_object_unref (store);
-}
-
 static void
 preferences_widget_sync_bool (const gchar *key, GtkWidget *widget)
 {
@@ -647,6 +607,7 @@ preferences_widget_sync_bool (const gchar *key, GtkWidget *widget)
 	}
 }
 
+#if 0
 static void
 preferences_widget_sync_string (const gchar *key, GtkWidget *widget)
 {
@@ -656,18 +617,18 @@ preferences_widget_sync_string (const gchar *key, GtkWidget *widget)
 		if (GTK_IS_ENTRY (widget)) {
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 		} else if (GTK_IS_RADIO_BUTTON (widget)) {
-			if (strcmp (key, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM) == 0) {
+			if (!tp_strdiff (key, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM)) {
 				GType        type;
 				GEnumClass  *enum_class;
 				GEnumValue  *enum_value;
 				GSList      *list;
 				GtkWidget   *toggle_widget;
-				
+
 				/* Get index from new string */
 				type = empathy_contact_list_store_sort_get_type ();
 				enum_class = G_ENUM_CLASS (g_type_class_peek (type));
 				enum_value = g_enum_get_value_by_nick (enum_class, value);
-				
+
 				if (enum_value) {
 					list = gtk_radio_button_get_group (GTK_RADIO_BUTTON (widget));
 					toggle_widget = g_slist_nth_data (list, enum_value->value);
@@ -682,48 +643,6 @@ preferences_widget_sync_string (const gchar *key, GtkWidget *widget)
 	}
 }
 
-static void
-preferences_widget_sync_string_combo (const gchar *key, GtkWidget *widget)
-{
-	gchar        *value;
-	GtkTreeModel *model;
-	GtkTreeIter   iter;
-	gboolean      found;
-
-	if (!empathy_conf_get_string (empathy_conf_get (), key, &value)) {
-		return;
-	}
-
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
-
-	found = FALSE;
-	if (value && gtk_tree_model_get_iter_first (model, &iter)) {
-
-		do {
-			gchar *name;
-			gtk_tree_model_get (model, &iter,
-					    COL_COMBO_NAME, &name,
-					    -1);
-
-			if (strcmp (name, value) == 0) {
-				found = TRUE;
-				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
-			}
-
-			g_free (name);
-		} while (!found && gtk_tree_model_iter_next (model, &iter));
-	}
-
-	/* Fallback to the first one. */
-	if (!found) {
-		if (gtk_tree_model_get_iter_first (model, &iter)) {
-			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
-		}
-	}
-
-	g_free (value);
-}
-
 static void
 preferences_notify_string_cb (EmpathyConf  *conf,
 			      const gchar *key,
@@ -731,14 +650,7 @@ preferences_notify_string_cb (EmpathyConf  *conf,
 {
 	preferences_widget_sync_string (key, user_data);
 }
-
-static void
-preferences_notify_string_combo_cb (EmpathyConf  *conf,
-				    const gchar *key,
-				    gpointer     user_data)
-{
-	preferences_widget_sync_string_combo (key, user_data);
-}
+#endif
 
 static void
 preferences_notify_bool_cb (EmpathyConf  *conf,
@@ -778,7 +690,7 @@ preferences_notify_int_cb (EmpathyConf  *conf,
 			   const gchar *key,
 			   gpointer     user_data)
 {
-	preferences_widget_sync_int (key, user_data);	
+	preferences_widget_sync_int (key, user_data);
 }
 
 static void
@@ -886,6 +798,7 @@ preferences_hookup_toggle_button (EmpathyPreferences *preferences,
 	}
 }
 
+#if 0
 static void
 preferences_hookup_radio_button (EmpathyPreferences *preferences,
 				 const gchar       *key,
@@ -915,32 +828,7 @@ preferences_hookup_radio_button (EmpathyPreferences *preferences,
 		preferences_add_id (preferences, id);
 	}
 }
-
-static void
-preferences_hookup_string_combo (EmpathyPreferences *preferences,
-				 const gchar       *key,
-				 GtkWidget         *widget)
-{
-	guint id;
-
-	preferences_widget_sync_string_combo (key, widget);
-
-	g_object_set_data_full (G_OBJECT (widget), "key",
-				g_strdup (key), g_free);
-
-	g_signal_connect (widget,
-			  "changed",
-			  G_CALLBACK (preferences_string_combo_changed_cb),
-			  NULL);
-
-	id = empathy_conf_notify_add (empathy_conf_get (),
-				      key,
-				      preferences_notify_string_combo_cb,
-				      widget);
-	if (id) {
-		preferences_add_id (preferences, id);
-	}
-}
+#endif
 
 static void
 preferences_hookup_sensitivity (EmpathyPreferences *preferences,
@@ -976,6 +864,7 @@ preferences_toggle_button_toggled_cb (GtkWidget *button,
 			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
 }
 
+#if 0
 static void
 preferences_radio_button_toggled_cb (GtkWidget *button,
 				     gpointer   user_data)
@@ -989,19 +878,19 @@ preferences_radio_button_toggled_cb (GtkWidget *button,
 
 	key = g_object_get_data (G_OBJECT (button), "key");
 
-	if (key && strcmp (key, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM) == 0) {
+	if (!tp_strdiff (key, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM)) {
 		GSList      *group;
 		GType        type;
 		GEnumClass  *enum_class;
 		GEnumValue  *enum_value;
-		
+
 		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-		
+
 		/* Get string from index */
 		type = empathy_contact_list_store_sort_get_type ();
 		enum_class = G_ENUM_CLASS (g_type_class_peek (type));
 		enum_value = g_enum_get_value (enum_class, g_slist_index (group, button));
-		
+
 		if (!enum_value) {
 			g_warning ("No GEnumValue for EmpathyContactListSort with GtkRadioButton index:%d",
 				   g_slist_index (group, button));
@@ -1009,30 +898,196 @@ preferences_radio_button_toggled_cb (GtkWidget *button,
 		}
 
 		value = enum_value->value_nick;
+	} else if (!tp_strdiff (key, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM)) {
+           return;
 	}
 
 	empathy_conf_set_string (empathy_conf_get (), key, value);
 }
+#endif
 
 static void
-preferences_string_combo_changed_cb (GtkWidget *combo,
-				     gpointer   user_data)
+preferences_theme_notify_cb (EmpathyConf *conf,
+			     const gchar *key,
+			     gpointer     user_data)
+{
+	EmpathyPreferences *preferences = user_data;
+	GtkComboBox        *combo;
+	gchar              *conf_name;
+	gchar              *conf_path;
+	GtkTreeModel       *model;
+	GtkTreeIter         iter;
+	gboolean            found = FALSE;
+
+	if (!empathy_conf_get_string (conf, EMPATHY_PREFS_CHAT_THEME, &conf_name)) {
+		return;
+	}
+
+	if (!empathy_conf_get_string (conf, EMPATHY_PREFS_CHAT_ADIUM_PATH, &conf_path)) {
+		g_free (conf_name);
+		return;
+	}
+
+	combo = GTK_COMBO_BOX (preferences->combobox_chat_theme);
+	model = gtk_combo_box_get_model (combo);
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		gboolean is_adium;
+		gchar *name;
+		gchar *path;
+
+		do {
+			gtk_tree_model_get (model, &iter,
+					    COL_COMBO_IS_ADIUM, &is_adium,
+					    COL_COMBO_NAME, &name,
+					    COL_COMBO_PATH, &path,
+					    -1);
+
+			if (!tp_strdiff (name, conf_name)) {
+				if (tp_strdiff (name, "adium") ||
+				    !tp_strdiff (path, conf_path)) {
+					found = TRUE;
+					gtk_combo_box_set_active_iter (combo, &iter);
+					g_free (name);
+					g_free (path);
+					break;
+				}
+			}
+
+			g_free (name);
+			g_free (path);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+	/* Fallback to the first one. */
+	if (!found) {
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			gtk_combo_box_set_active_iter (combo, &iter);
+		}
+	}
+
+	g_free (conf_name);
+	g_free (conf_path);
+}
+
+static void
+preferences_theme_changed_cb (GtkComboBox        *combo,
+			      EmpathyPreferences *preferences)
 {
-	const gchar  *key;
 	GtkTreeModel *model;
 	GtkTreeIter   iter;
+	gboolean      is_adium;
 	gchar        *name;
+	gchar        *path;
 
-	key = g_object_get_data (G_OBJECT (combo), "key");
-
-	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter)) {
-		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+	if (gtk_combo_box_get_active_iter (combo, &iter)) {
+		model = gtk_combo_box_get_model (combo);
 
 		gtk_tree_model_get (model, &iter,
+				    COL_COMBO_IS_ADIUM, &is_adium,
 				    COL_COMBO_NAME, &name,
+				    COL_COMBO_PATH, &path,
 				    -1);
-		empathy_conf_set_string (empathy_conf_get (), key, name);
+
+		empathy_conf_set_string (empathy_conf_get (),
+					 EMPATHY_PREFS_CHAT_THEME,
+					 name);
+		if (is_adium == TRUE)
+			empathy_conf_set_string (empathy_conf_get (),
+						 EMPATHY_PREFS_CHAT_ADIUM_PATH,
+						 path);
 		g_free (name);
+		g_free (path);
+	}
+}
+
+static void
+preferences_themes_setup (EmpathyPreferences *preferences)
+{
+	GtkComboBox   *combo;
+	GtkCellLayout *cell_layout;
+	GtkCellRenderer *renderer;
+	GtkListStore  *store;
+	const gchar  **themes;
+	GList         *adium_themes;
+	gint           i;
+	guint          id;
+
+	combo = GTK_COMBO_BOX (preferences->combobox_chat_theme);
+	cell_layout = GTK_CELL_LAYOUT (combo);
+
+	/* Create the model */
+	store = gtk_list_store_new (COL_COMBO_COUNT,
+				    G_TYPE_BOOLEAN, /* Is an Adium theme */
+				    G_TYPE_STRING,  /* Display name */
+				    G_TYPE_STRING,  /* Theme name */
+				    G_TYPE_STRING); /* Theme path */
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+		COL_COMBO_VISIBLE_NAME, GTK_SORT_ASCENDING);
+
+	/* Fill the model */
+	themes = empathy_theme_manager_get_themes ();
+	for (i = 0; themes[i]; i += 2) {
+		gtk_list_store_insert_with_values (store, NULL, -1,
+			COL_COMBO_IS_ADIUM, FALSE,
+			COL_COMBO_VISIBLE_NAME, _(themes[i + 1]),
+			COL_COMBO_NAME, themes[i],
+			COL_COMBO_PATH, NULL,
+			-1);
+	}
+
+	adium_themes = empathy_theme_manager_get_adium_themes ();
+	while (adium_themes != NULL) {
+		GHashTable *info;
+		const gchar *name;
+		const gchar *path;
+
+		info = adium_themes->data;
+		name = tp_asv_get_string (info, "CFBundleName");
+		path = tp_asv_get_string (info, "path");
+
+		if (name != NULL && path != NULL) {
+			gtk_list_store_insert_with_values (store, NULL, -1,
+				COL_COMBO_IS_ADIUM, TRUE,
+				COL_COMBO_VISIBLE_NAME, name,
+				COL_COMBO_NAME, "adium",
+				COL_COMBO_PATH, path,
+				-1);
+		}
+		g_hash_table_unref (info);
+		adium_themes = g_list_delete_link (adium_themes, adium_themes);
+	}
+
+	/* Add cell renderer */
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (cell_layout, renderer, TRUE);
+	gtk_cell_layout_set_attributes (cell_layout, renderer,
+		"text", COL_COMBO_VISIBLE_NAME, NULL);
+
+	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
+	g_object_unref (store);
+
+	g_signal_connect (combo, "changed",
+			  G_CALLBACK (preferences_theme_changed_cb),
+			  preferences);
+
+	/* Select the theme from the gconf key and track changes */
+	preferences_theme_notify_cb (empathy_conf_get (),
+				     EMPATHY_PREFS_CHAT_THEME,
+				     preferences);
+	id = empathy_conf_notify_add (empathy_conf_get (),
+				      EMPATHY_PREFS_CHAT_THEME,
+				      preferences_theme_notify_cb,
+				      preferences);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
+
+	id = empathy_conf_notify_add (empathy_conf_get (),
+				      EMPATHY_PREFS_CHAT_ADIUM_PATH,
+				      preferences_theme_notify_cb,
+				      preferences);
+	if (id) {
+		preferences_add_id (preferences, id);
 	}
 }
 
@@ -1067,6 +1122,7 @@ empathy_preferences_show (GtkWindow *parent)
 	static EmpathyPreferences *preferences;
 	GtkBuilder                *gui;
 	gchar                     *filename;
+	GtkWidget                 *page;
 
 	if (preferences) {
 		gtk_window_present (GTK_WINDOW (preferences->dialog));
@@ -1079,21 +1135,25 @@ empathy_preferences_show (GtkWindow *parent)
 	gui = empathy_builder_get_file (filename,
 		"preferences_dialog", &preferences->dialog,
 		"notebook", &preferences->notebook,
-		"checkbutton_show_avatars", &preferences->checkbutton_show_avatars,
-		"checkbutton_compact_contact_list", &preferences->checkbutton_compact_contact_list,
 		"checkbutton_show_smileys", &preferences->checkbutton_show_smileys,
+		"checkbutton_show_contacts_in_rooms", &preferences->checkbutton_show_contacts_in_rooms,
 		"combobox_chat_theme", &preferences->combobox_chat_theme,
 		"checkbutton_separate_chat_windows", &preferences->checkbutton_separate_chat_windows,
 		"checkbutton_autoconnect", &preferences->checkbutton_autoconnect,
-		"radiobutton_contact_list_sort_by_name", &preferences->radiobutton_contact_list_sort_by_name,
-		"radiobutton_contact_list_sort_by_state", &preferences->radiobutton_contact_list_sort_by_state,
 		"checkbutton_notifications_enabled", &preferences->checkbutton_notifications_enabled,
 		"checkbutton_notifications_disabled_away", &preferences->checkbutton_notifications_disabled_away,
 		"checkbutton_notifications_focus", &preferences->checkbutton_notifications_focus,
+		"checkbutton_notifications_contact_signin", &preferences->checkbutton_notifications_contact_signin,
+		"checkbutton_notifications_contact_signout", &preferences->checkbutton_notifications_contact_signout,
 		"checkbutton_sounds_enabled", &preferences->checkbutton_sounds_enabled,
 		"checkbutton_sounds_disabled_away", &preferences->checkbutton_sounds_disabled_away,
 		"treeview_sounds", &preferences->treeview_sounds,
 		"treeview_spell_checker", &preferences->treeview_spell_checker,
+		"checkbutton_location_publish", &preferences->checkbutton_location_publish,
+		"checkbutton_location_reduce_accuracy", &preferences->checkbutton_location_reduce_accuracy,
+		"checkbutton_location_resource_network", &preferences->checkbutton_location_resource_network,
+		"checkbutton_location_resource_cell", &preferences->checkbutton_location_resource_cell,
+		"checkbutton_location_resource_gps", &preferences->checkbutton_location_resource_gps,
 		NULL);
 	g_free (filename);
 
@@ -1118,12 +1178,18 @@ empathy_preferences_show (GtkWindow *parent)
 	preferences_sound_load (preferences);
 
 	if (empathy_spell_supported ()) {
-		GtkWidget *page;
-
 		page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences->notebook), 2);
 		gtk_widget_show (page);
 	}
 
+	page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences->notebook), 3);
+#if HAVE_GEOCLUE
+	gtk_widget_show (page);
+#else
+	gtk_widget_hide (page);
+#endif
+
+
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog),
 					      GTK_WINDOW (parent));
